#!/bin/bash

# On ubuntu you need to install libqt4-core:i386 and libqt4-gui:i386 to run this script

top_repo_dir=$(dirname $(dirname $(dirname $(dirname $(readlink -f $0)))))

# Cleanup previous builds
rm -rf $top_repo_dir/build/toFlash/*
mkdir -p $top_repo_dir/build/toFlash

# Copy boot partition (contains kernel and ramdisk)
cp $top_repo_dir/build/tmp/deploy/images/edison/edison-image-edison.hddimg $top_repo_dir/build/toFlash/

# Copy u-boot
cp $top_repo_dir/build/tmp/deploy/images/edison/u-boot-edison.img $top_repo_dir/build/toFlash/
cp $top_repo_dir/build/tmp/deploy/images/edison/u-boot-edison.bin $top_repo_dir/build/toFlash/

# Copy u-boot environments files binary
cp -R $top_repo_dir/build/tmp/deploy/images/edison/u-boot-envs $top_repo_dir/build/toFlash

# Copy IFWI
cp $top_repo_dir/device-software/utils/flash/ifwi/mcg_sku/*.bin $top_repo_dir/build/toFlash/
cp $top_repo_dir/device-software/utils/flash/ifwi/edison/*.bin $top_repo_dir/build/toFlash/

# build Ifwi file for using in DFU mode
# Remove FUP footer (144 bytes) as it's not needed when we directly write to boot partitions
dd if=$top_repo_dir/build/toFlash/edison_ifwi-dbg.bin of=$top_repo_dir/build/toFlash/edison_ifwi-dbg-dfu.bin bs=4194304 count=1
dd if=$top_repo_dir/build/toFlash/ifwi_saltbay_pr2.bin of=$top_repo_dir/build/toFlash/ifwi_saltbay_pr2-dfu.bin bs=4194304 count=1

# Copy rootfs
cp $top_repo_dir/build/tmp/deploy/images/edison/edison-image-edison.ext4 $top_repo_dir/build/toFlash/

# Copy symbols files
mkdir -p $top_repo_dir/build/symbols
cp $top_repo_dir/build/tmp/deploy/images/edison/vmlinux $top_repo_dir/build/symbols/
cp $top_repo_dir/build/tmp/deploy/images/edison/u-boot-edison.bin $top_repo_dir/build/symbols/


# copy phoneflashtool xml file
cp $top_repo_dir/device-software/utils/flash/pft-config-mcg_sku.xml $top_repo_dir/build/toFlash/
cp $top_repo_dir/device-software/utils/flash/pft-config-edison.xml $top_repo_dir/build/toFlash/

# Copy flashing script
cp $top_repo_dir/device-software/utils/flash/flashall.sh $top_repo_dir/build/toFlash/
cp $top_repo_dir/device-software/utils/flash/flashall.bat $top_repo_dir/build/toFlash/

# Generates a formatted list of all packages included in the image
awk '{print $1 " " $3}' $top_repo_dir/build/tmp/deploy/images/edison/edison-image-edison.manifest > $top_repo_dir/build/toFlash/package-list.txt

echo "**** Done ***"
echo "Files ready to flash in $top_repo_dir/build/toFlash/"
echo "Run the flashall script there to start flashing."
echo "*************"
